data {

    int<lower=0> N_SPECIES; //integer lower bound equals 0
    int<lower=0> N_SUBJECTS; 
    int<lower=0> N_STUDIES; 
    
    int<lower=0> N_ROWS;
    int<lower=0> N_TASKS;
    
    int subject[N_ROWS]; // which subject is each row
    int species[N_ROWS]; // which species
    int study[N_ROWS];   // which study
    int task[N_ROWS];    // which task
    
    int n1[N_ROWS];
    int n2[N_ROWS];
    int ncorrect[N_ROWS];
    int ntrials[N_ROWS];
    
    // Covariance matrix passed in
    cov_matrix[N_SPECIES] phylocov;
}

transformed data { }

parameters {

    real B0; // for generating the scatterplots there are no predictors included

    // parameters for phylogenetic regression
    real<lower=0,upper=1> lambda_w;

    // Define the random effects
    // NOTE: These effects are not grouped (e.g. within task)
    vector[N_SPECIES]  species_effect; // vector since it's sampled from a matrix
    real               study_effect[N_STUDIES];
    real               task_effect[N_TASKS];
    real               subject_effect[N_SUBJECTS];
   
    real<lower=0> species_scale;
    real<lower=0> study_scale;
    real<lower=0> task_scale;
    real<lower=0> subject_scale;
    
}
    
model {    
    vector[N_SPECIES] mu_w; // the species means
    
    int imputed_index = 1;// index into missing arrays
                
    // priors
	lambda_w  ~ uniform(0,1);
	
    study_effect   ~ normal(0,1);
    task_effect    ~ normal(0,1);
    subject_effect ~ normal(0,1);    
    // species effect is defined by the normal below
    
    species_scale ~ exponential(1);
    study_scale   ~ exponential(1);
    task_scale    ~ exponential(1);
    subject_scale ~ exponential(1);
    
    // This is the beta for intercept
    B0 ~ normal(-1,1); // the mean value of log(w)
	
	for(i in 1:N_SPECIES) { // fill in the missing data with the inferred
		
		// and compute the species expected mean
		mu_w[i] = B0;
	}	
	
	// note that the species_scale handles the scaling of these
	species_effect ~ multi_normal(rep_vector(0.0, N_SPECIES), 
								  phylocov*lambda_w + (1-lambda_w)*diag_matrix(rep_vector(1.0, N_SPECIES))); 
	
	// Now the likelihood:       
    for(i in 1:N_ROWS) {
        // This is the Weber ratio
        real w = exp(mu_w[species[i]]
                     + species_effect[species[i]]*species_scale
                     + subject_effect[subject[i]]*subject_scale
                     + study_effect[study[i]]*study_scale
                     + task_effect[task[i]]*task_scale
					);

        // binomial likelihood
        ncorrect[i] ~ binomial(ntrials[i], Phi(abs(n1[i]-n2[i]) / (w * sqrt(n1[i]^2 + n2[i]^2))));
    }

}

generated quantities {
	
	vector[N_SPECIES] species_W;
	
	{ // recompute mu_w because stan limitation
		int imputed_index = 1;
	    vector[N_SPECIES] mu_w; // the species means
		for(i in 1:N_SPECIES) { // fill in the missing data with the inferred
		
			// and compute the species expected mean
			mu_w[i] = B0;
		}	
	
		for(s in 1:N_SPECIES) {
			// compute average species value (at 0 subject effect, 0 study effect, 0 task effect)
			// different tasks, 1 is array, 2 is sequential, 3 is simultaneous
			species_W[s] = exp(mu_w[s] + species_effect[s]*species_scale + task_effect[3]*task_scale); 
		}
	}
}






