data {

    int<lower=0> N_SPECIES; //integer lower bound equals 0
    int<lower=0> N_SUBJECTS; 
    int<lower=0> N_STUDIES; 
    
    int<lower=0> N_ROWS;
    int<lower=0> N_TASKS;
    
    int subject[N_ROWS]; // which subject is each row
    int species[N_ROWS]; // which species
    int study[N_ROWS];   // which study
    int task[N_ROWS];    // which task
    
    int sentinel; // what value indicates missing data?    
    
    real X1[N_SPECIES]; // the predictors - should be z-scored (so prior on missing makes sense)
    //X1 is one dimensional array of real (continuous) values of array size Number of species
    
    int<lower=0> X1NMissing; // number of missing data points
    
    int n1[N_ROWS];
    int n2[N_ROWS];
    int ncorrect[N_ROWS];
    int ntrials[N_ROWS];
    
    // Covariance matrix passed in
    cov_matrix[N_SPECIES] phylocov;
}

transformed data { }

parameters {

    real B0; // this is the mean log W
    real B1; 

    // parameters for phylogenetic regression
    real<lower=0,upper=1> lambda_w;

    // Define the random effects
    // NOTE: These effects are not grouped (e.g. within task)
    vector[N_SPECIES]  species_effect; // vector since it's sampled from a matrix
    real               study_effect[N_STUDIES];
    real               task_effect[N_TASKS];
    real               subject_effect[N_SUBJECTS];
   
    real<lower=0> species_scale;
    real<lower=0> study_scale;
    real<lower=0> task_scale;
    real<lower=0> subject_scale;
    
    // imputed values
    real X1imputed[X1NMissing];  
}
    
model {    
    vector[N_SPECIES] mu_w; // the species means
    
    int imputed_index = 1;// index into missing arrays
                
    // priors
	lambda_w  ~ uniform(0,1);
	
    study_effect   ~ normal(0,1);
    task_effect    ~ normal(0,1);
    subject_effect ~ normal(0,1);    
    // species effect is defined by the normal below
    
    species_scale ~ exponential(1);
    study_scale   ~ exponential(1);
    task_scale    ~ exponential(1);
    subject_scale ~ exponential(1);
	
    // This is the beta for the predictor x
    B0 ~ normal(-1,1); // the mean value of log(w)
	B1 ~ normal(0,5); 
	
	// Generate the predictions here from the predictor
	X1imputed ~ normal(0,1); // prior on missing data -- NOTE this therefore assumes data was z-scored, so imputed are standardized
	for(i in 1:N_SPECIES) { // fill in the missing data with the inferred
		real x; // what is the x value?
		
		// fill in the predictor either with the measured value X1 or with the inferred value X1Missing if X1 contains the sentinel
		x = X1[i];
		if(X1[i] == sentinel) {
            x = X1imputed[imputed_index];
            imputed_index = imputed_index+1;
		}
		
		// and compute the species expected mean
	  mu_w[i] = B0 + B1*x;
	}	
	
	// note that the species_scale handles the scaling of these
	species_effect ~ multi_normal(rep_vector(0.0, N_SPECIES), 
								  phylocov*lambda_w + (1-lambda_w)*diag_matrix(rep_vector(1.0, N_SPECIES))); 
	
	// Now the likelihood:       
    for(i in 1:N_ROWS) {
        // This is the Weber ratio
        real w = exp(mu_w[species[i]]
                     + species_effect[species[i]]*species_scale
                     + subject_effect[subject[i]]*subject_scale
                     + study_effect[study[i]]*study_scale
                     + task_effect[task[i]]*task_scale
					);

        // binomial likelihood, Phi is the probit function in Stan, the CDF of the standardised Normal N(0,1)  
        ncorrect[i] ~ binomial(ntrials[i], Phi(abs(n1[i]-n2[i]) / (w * sqrt(n1[i]^2 + n2[i]^2))));
        
    }

}
