library(rstan)
library(foreign)
library(ape)
library(ggplot2)
library(bayesplot)

rstan_options(auto_write = TRUE)
#options(width=Sys.getenv("COLUMNS"))  # fix the number of columns
    
# a value used for missing data -- when this occurs, 
# stan knows that the data point must be imputed
sentinel = -9999;

t <- read.nexus("WSpeciesTree.nex")  
t$edge.length = t$edge.length / max(t$edge.length) 
phylocov=vcv(t)

d = read.csv('NumberData.csv', header = TRUE)
d = subset(d, species <= 33)

s <- read.csv('SpeciesPredictors.csv')

D <- NULL
# mycol="rbv.no"
for(mycol in c("endocranial","residualbrain","logcortneurons",
               "frugivory","logpopsize","loghomerange","logdayjourney","selfcontrol2",
               "gencogrev","logcortneurdens","logcerebneurdens",
               "logcerebneurons")) {

	x = c(scale(s[,mycol]))
	x[is.na(x)] = sentinel # replace NAs with our numerical sentinel AFTER z-scoring
	
	# Construct the data to send
	data <- list(
        sentinel=sentinel,
        N_SPECIES=length(unique(d$species)),
        N_STUDIES=length(unique(d$study)),
        N_SUBJECTS=length(unique(d$subject)),
        N_ROWS=nrow(d),
        N_TASKS=3,
        subject=d$subject,
        species=d$species,
        study=d$study,
        task=d$task,
        n1=d$n1,
        n2=d$n2,
        ncorrect=as.integer(d$ncorrect),
        ntrials=d$ntrials,
        X1=x,
        X1NMissing=sum(x==sentinel),
        phylocov=phylocov
	) 

	m <- stan_model('model2.stan') #added to prevent error 'recompiling to avoid crashing R'
	fit <- sampling(m, data=data, iter=5000, thin=5, chains=4, cores=6, pars=c("subject_effect"), include=FALSE, control=list(adapt_delta=0.999, max_treedepth=15))
	print(fit)
	saveRDS(fit, file = "fit.rds")
	
	# main predictors of interest here
	b0v <- extract(fit, "B0")$B0
	b1v <- extract(fit, "B1")$B1
	lv <- extract(fit, "lambda_w")$lambda_w

	D <- rbind(D, data.frame(predictor=mycol, 
	                         b0.5=quantile(b0v,0.05), b0.25=quantile(b0v,0.25), b0.50=quantile(b0v,.5), b0.75=quantile(b0v,0.75), b0.95=quantile(b0v,0.95),
	                         b1.5=quantile(b1v,0.05), b1.25=quantile(b1v,0.25), b1.50=quantile(b1v,.5), b1.75=quantile(b1v,0.75), b1.95=quantile(b1v,0.95),
	                         l.5=quantile(lv,0.05), l.25=quantile(lv,0.25), l.50=quantile(lv,.5), l.75=quantile(lv,0.75), l.95=quantile(lv,0.95)
                                 ))
	write.csv(D, "Dall.csv", row.names=TRUE)
	posterior <- as.array(fit)
	pdf_file_name <- paste("intervall_", mycol, ".pdf", sep="")
	pdf(pdf_file_name)
	color_scheme_set("blue")
	plot(mcmc_intervals(posterior, pars = c("B0", "B1", "lambda_w")))
	dev.off()
	posterior <- as.array(fit)
	pdf_file_name <- paste("histall_", mycol, ".pdf", sep="")
	pdf(pdf_file_name)
	color_scheme_set("blue")
	plot(mcmc_hist(posterior, pars = c("B0", "B1", "lambda_w")))
	dev.off()
	
}


# d <- read.csv("Dall.csv", header=T)
# 
# #B1 posterior uncertainty intervals by predictor
# ggplot(d, aes(x=predictor, y=b1.50, color=Category)) +
#   scale_color_manual(values=c("black", "purple","olivedrab")) +
#   scale_x_discrete(limits = c("endocranial","residualbrain","logcortneurons","logcerebneurons","logcortneurdens","logcerebneurdens","frugivory","logpopsize","loghomerange","logdayjourney","gencogrev","selfcontrol2")) +
#   geom_hline(yintercept=0, color="darkblue") +
#   geom_errorbar(aes(ymin=b1.25, ymax=b1.75), size=0.8,width=0.2) +
#   geom_errorbar(aes(ymin=b1.5,  ymax=b1.95), size=0.8, width=0.0, alpha=0.4) +    
#   geom_point(size=3) +
#   theme_classic() + 
#   theme(axis.text.x=element_text(size=12,angle=90,hjust=1),
#         legend.text = element_text(size=12),
#         legend.title = element_text(size=12),
#         axis.title.x=element_text(size=14),
#         axis.title.y=element_text(size=14)) +
#   xlab("Predictor") + ylab("Coefficient") + ylim(-0.5, 0.5)
